import re
import argparse
import pandas as pd
import numpy as np
from openpyxl.utils import get_column_letter

parser = argparse.ArgumentParser('Export discovered hosts to excel')
parser.add_argument('-r', '--drule',  help='discovery rule')
parser.add_argument('-o', '--output', help='output save to an excel file, xx.xlsx')
parser.set_defaults(handler=lambda args: main(args))

# 设置excel自动列宽
def to_excel_auto_column_weight(df, writer, sheet_name="Sheet1"):
    """
    DataFrame保存为excel并自动设置列宽
    """
    df.to_excel(writer, sheet_name=sheet_name, index=False)
    # 计算每列表头的字符宽度
    column_widths = (
        df.columns.to_series().apply(lambda x: len(x.encode('utf8'))).values
    )
    # 计算每列的最大字符宽度
    max_widths = (
        df.astype(str).applymap(lambda x: len(x.encode('utf8'))).agg(max).values
    )

    widths = np.max([column_widths, max_widths], axis=0)
    worksheet = writer.sheets[sheet_name]
    for i, width in enumerate(widths, 1):
        # openpyxl设置字符宽度时会缩水0.5左右个字符，所以+2使左右都空出一个字宽。
        worksheet.column_dimensions[get_column_letter(i)].width = width + 2

def main(args):

    zapi = args.zapi
    rules = zapi.drule.get({
        'output': 'extend',
        'selectDChecks': 'extend',
        'selectDHosts': 'extend',
        'search': {
            'name': args.drule
        },
        'searchWildcardsEnabled': True,
    })

    icmp_ping_check_ids = [check['dcheckid'] for rule in rules for check in rule['dchecks'] if check['type'] == '12']
    zabbix_agent_check_ids = [check['dcheckid'] for rule in rules for check in rule['dchecks'] if check['type'] == '9']

    # 正则匹配总类-业务网类-负责人
    pattern = re.compile(r'([^-]+)-([^-]+)-([^-]+)')

    df = pd.DataFrame(columns=['dhostid', 'check_type','ipv4', 'monitored', 'host', 'status', 'multi_inf', 'os', 'net', 'poc'])

    # 获取 Hypervisors 组内host name
    hyperv_grp = zapi.hostgroup.get({'filter': {'name':'Hypervisors'},'selectHosts': ['name']})
    hyperv_hosts = [h['name'] for grp in hyperv_grp for h in grp['hosts']]
    
    # 获取 dservice \ drule name \ Host
    dservices = zapi.dservice.get({
        'output': ['dcheckid', 'ip', 'status', 'value', 'dhostid'],
        'druleids': [rule['druleid'] for rule in rules],
        'selectDRules': ['name'],
        'selectHosts': ['host', 'status'],
    })


    for dservice in dservices:

        info = {}

        # icmp check
        if dservice['dcheckid'] in icmp_ping_check_ids:

            info['dhostid'] = dservice['dhostid']
            info['check_type'] = 'icmp'
            info['ipv4'] = dservice['ip']

            if dservice['hosts']:
                info['monitored'] = '是'     # 是否被监控
                info['host'] = dservice['hosts'][0]['host'] # 监控主机\状态
                info['status'] = ('启用' if dservice['hosts'][0]['status'] == '0' else '禁用')

            elif dservice['ip'] in hyperv_hosts:
                t = zapi.host.get({'filter': {'name': dservice['ip']}, 'selectInventory': ['poc_1_name', 'os_short']})
                zbx_host = t[0]
                info['monitored'] = '是'
                info['host'] = zbx_host['host']
                info['status'] = ('启用' if zbx_host['status'] == '0' else '禁用')
                info['poc'] = zbx_host['inventory'].get('poc_1_name')
                info['os'] = zbx_host['inventory'].get('os_short')


        # agent check
        if dservice['dcheckid'] in zabbix_agent_check_ids:

            info['dhostid'] = dservice['dhostid']
            info['check_type'] = 'agent'
            info['ipv4'] = dservice['ip']

            # 验证是否在zabbix host中
            t = zapi.host.get({'filter': {'host': dservice['value']}, 'selectInventory': ['poc_1_name', 'os_short']})
            
            if t:
                zbx_host = t[0]
                info['host'] = dservice['value']
                info['monitored'] = '是'
                info['status'] = ('启用' if dservice['status'] == '0' else '禁用')  
                info['poc'] = zbx_host['inventory'].get('poc_1_name')
                info['os'] = zbx_host['inventory'].get('os_short')
          
        if info:
            # rule name符合"总类-业务网类-负责人"形式，提取出业务网络和负责人信息
            drule_name = dservice['drules'][0]['name']
            if pattern.search(drule_name):
                _, net, poc = pattern.findall(drule_name)[0]

                # 如从inventory中取到了poc, 则优先使用, 否则使用rule name中的负责人
                info['poc'] = info.get('poc') and info['poc'] or poc
                info['net'] = net

            row = pd.Series(info)
            df = df.append(row, ignore_index=True)


    # 既有icmp check 又有agent check的情况, dhostid相同, 通过check_type排序后, 
    # 去除icmp check的那一行数据, 以agent check为准
    df = df.sort_values(['dhostid', 'check_type'], ascending=False).drop_duplicates(subset='dhostid', keep='last')

    # 按照host进行group by, 其余字段进行单元格合并, (仅host不为空的行参与)
    # 如果同一host有多个不同ipv4, 则认为是多网卡(有可能是一张物理网卡使用多个ip)
    df2 = df.groupby('host', as_index=False).apply(lambda x: pd.Series({
        'ipv4': ','.join(x.ipv4.unique()),
        'monitored': ','.join([i for i in x.monitored.unique() if isinstance(i, str)]),
        'status': ','.join([i for i in x.status.unique() if isinstance(i, str)]),
        'multi_inf': ('是' if x.ipv4.count() > 1 else '否'),
        'net': ','.join([i for i in x.net.unique() if isinstance(i, str)]),
        'poc': ','.join([i for i in x.poc.unique() if isinstance(i, str)]),
        'os': ','.join([i for i in x.os.unique() if isinstance(i, str)]),
    }))


    # 将df中host为空的数据与 df2拼接在一起
    res = df[df.host.isna()].drop(['dhostid', 'check_type'], axis=1).append(df2).reset_index(drop=True)
    res.sort_values(by=['host'], na_position='last', inplace=True)
    res.monitored.fillna('否', inplace=True)
    res.multi_inf.fillna('否', inplace=True)

    # 字段重命名为中文
    res.rename(columns={
        'ipv4': 'ipv4(s)',
        'host': '监控主机',
        'monitored': '是否监控',
        'status': '监控状态',
        'multi_inf': '是否多网卡',
        'net': '网段名称',
        'poc': '负责人',
        'os': '操作系统',
    }, inplace=True)

    # 设置屏幕输出的参数
    pd.set_option('display.width', 180)
    pd.set_option('display.max_rows', None)
    pd.set_option('display.max_columns', 99)
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    print(res)

    if args.output:
        fname = args.output.endswith('.xlsx') and args.output or args.output + '.xlsx'
        writer = pd.ExcelWriter(fname)
        to_excel_auto_column_weight(res, writer, 'discovery数据')
        writer.save()



#!/bin/env python3
# coding:utf-8

"""
从vCenter中获取esxi主机信息，更新到Zabbix inventory
"""
import sys
import argparse
import logging
from urllib.parse import urlparse
from pyVmomi import vim, vmodl
from pyVim.connect import Disconnect, SmartConnectNoSSL
from infi.pyvmomi_wrapper.esxcli import EsxCLI
from zabbix_api import ZabbixAPIException

parser = argparse.ArgumentParser("Correct discoverd host name")
parser.add_argument('-l', '--limit', action='append', type=str, help="Specify hypervisors")
parser.set_defaults(handler=lambda args: main(args))


def fetch_esxi(host, user, pwd, esxi_name, port=443):
    '''
    通过 pyVmomi 模块读取 vCenter 中 esxi server的信息。
    '''
    si = SmartConnectNoSSL(host=host, user=user, pwd=pwd, port=port)
    content = si.RetrieveContent()
    container = content.viewManager.CreateContainerView(content.rootFolder, [vim.HostSystem, vim.Datacenter], True)

    # vcenter自定义的属性, 取系统中对应的字段的key, {'机柜': 43, '机房名称': 44}
    custom_key = {'机柜':'', '机房名称':''}

    cfm = content.customFieldsManager
    for field in cfm.field:
        if field.name in custom_key.keys():
            custom_key[field.name] = field.key

    esxi_info = {}

    # 取数据中心的机房名称
    for view in container.view:
        if not isinstance(view, vim.Datacenter):
            continue

        # esxi 属于某 datacenter, 则取 datacenter 自定义属性
        for child in view.hostFolder.childEntity:

            isFind = False  #是否在该 datacenter 找到目标 esxi

            # 子节点是 ClusterCompute, esxi 挂在 ClusterCompute下面
            if isinstance(child, vim.ClusterComputeResource):
                for host in child.host:
                    if esxi_name == host.name:
                        isFind = True
            else:
            # esxi 直接挂在 datacenter 下
                if esxi_name == child.name:
                    isFind = True

            if isFind:
                # location 默认值为datacenter的名字
                esxi_info['location'] = view.name
                # 如果datacenter有“机房名称”自定义属性，则覆盖前者。
                for prop in view.customValue:
                    if prop.key == custom_key['机房名称']:
                        esxi_info['location'] = prop.value
                        break


    # 取esxi属性
    for esxi_ in container.view:
        if esxi_.name != esxi_name:
            continue
        esxi_info['type'] = 'Server'
        esxi_info['name'] = esxi_.name
        esxi_info['os'] = esxi_.config.product.name
        esxi_info['os_full'] = esxi_.summary.config.product.fullName
        esxi_info['os_short'] = esxi_.config.product.osType
        esxi_info['tag'] = 'Esxi'
        esxi_info['model'] = esxi_.summary.hardware.model
        esxi_info['vendor'] = esxi_.summary.hardware.vendor
        esxi_info['management_ip'] = esxi_.summary.managementServerIp
        esxi_info['cpu'] = '(%s) * %s  %s核  %s线程' % (esxi_.summary.hardware.cpuModel,
                           esxi_.summary.hardware.numCpuPkgs,
                           esxi_.summary.hardware.numCpuCores,
                           esxi_.summary.hardware.numCpuThreads)
        esxi_info['mem'] = f'内存: {esxi_.summary.hardware.memorySize/1024/1024//1024}GB'

        # get host's ipA and netmask
        for vnic in esxi_.config.network.vnic:
            if isinstance(vnic, vim.host.VirtualNic):
                esxi_info['ip'] = vnic.spec.ip.ipAddress
                esxi_info['subnet_mask'] = vnic.spec.ip.subnetMask

        # get host's mac address
        for portgroup in esxi_.config.network.portgroup:
            for port in portgroup.port:
                if port.type == 'host':
                    esxi_info['mac_address'] = ''.join(port.mac)

        # get host's serial_number
        ordered_keys = ['SerialNumberTag', 'EnclosureSerialNumberTag', 'ServiceTag']
        for iden in esxi_.summary.hardware.otherIdentifyingInfo:
            if isinstance(iden, vim.host.SystemIdentificationInfo) and iden.identifierType.key in ordered_keys:
                esxi_info[iden.identifierType.key] = iden.identifierValue

        for key in ordered_keys:
            if esxi_info.get(key):
                esxi_info['serial_number'] = esxi_info.get(key)
                break

        if not esxi_info.get('serial_number'):
            cli = EsxCLI(esxi_)
            hardware = cli.get("hardware.platform")
            esxi_info['serial_number'] = hardware.Get().SerialNumber


        # 取机柜名称
        esxi_info['site_rack'] = None
        for prop in esxi_.customValue:
            if prop.key == custom_key['机柜']:
                esxi_info['site_rack'] = prop.value
                break

    Disconnect(si)
    return esxi_info


def main(args):
    '''
    更新 zabbix 中 Hypervisors 组中 host 的 inventory. 其内容从 vCenter 中获取.
    '''
    zapi = args.zapi
    
    zbx_group = zapi.hostgroup.get({
        'output': ['groupid', 'name'],
        'selectHosts': ['hostid', 'name'],
        'filter': {'name': 'Hypervisors'}
    })

    if not zbx_group:
        sys.exit()

    grp_hosts = zbx_group[0].get('hosts')

    # 如指定limit参数, 则仅处理列表中的host 
    if args.limit:
        grp_hosts = [h for h in grp_hosts if h['name'] in args.limit]


    # 调用zapi 查询host的macros信息
    hosts = zapi.host.get({
        'output': ['hostid', 'name'],
        'hostids': [h['hostid'] for h in grp_hosts],
        'selectMacros': ['macro', 'value']
    })

    for host in hosts:
        host_id = host['hostid']
        host_name = host['name']

        # 从zabbix host的macros 中取出对应的vcenter登录信息
        url = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$URL}'])
        vcenter_ip = urlparse(url).hostname
        vcenter_admin = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$USERNAME}'])
        vcenter_pwd = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$PASSWORD}'])

        logging.info(f'[fetch_esxi] vcenter_ip: {vcenter_ip}, esxi_name:{host_name}')

        try:
            esxi_info = fetch_esxi(host=vcenter_ip,
                                   user=vcenter_admin,
                                   pwd=vcenter_pwd,
                                   esxi_name=host_name)

            zapi.host.update({
                "hostid": host_id,
                "inventory": {
                    "type": esxi_info.get('type'),
                    "name": esxi_info.get('name'),
                    "alias": esxi_info.get('ip'),
                    "os": esxi_info.get('os'),
                    "os_full": esxi_info.get('os_full'),
                    "os_short": esxi_info.get('os_short'),
                    "serialno_a": esxi_info.get('serial_number'),
                    "tag": esxi_info.get('tag'),
                    "macaddress_a": esxi_info.get('mac_address'),
                    "macaddress_b": esxi_info.get('mac_address'),
                    "model": esxi_info.get('model'),
                    "vendor": esxi_info.get('vendor'),
                    "host_networks": esxi_info.get('ip'),
                    "host_netmask": esxi_info.get('subnet_mask'),
                    "hardware_full": '\n'.join([esxi_info.get('cpu',''), esxi_info.get('mem','')]),
                    "location": esxi_info.get('location'),
                    "site_rack": esxi_info.get('site_rack')
                }
            })
            logging.info(f'update host->{host_name} success!')

        except ZabbixAPIException as error:
            logging.exception(error)
        except vmodl.MethodFault as error:
            logging.exception(error.msg)
            continue



import multicommand
import logging
from zabbix_api import ZabbixAPI
from lib import commands


def main():
    parser = multicommand.create_parser(commands)
    parser.add_argument('-s', '--zbx-server', required=True, help='URL of zabbix server')
    parser.add_argument('-u', '--zbx-user', required=True, help='Zabbix server login username')
    parser.add_argument('-p', '--zbx-passwd', required=True, help='Zabbix server login password')
    parser.add_argument('-t', '--timeout', default=60, type=int, help='Zabbix API timeout')
    # parser.add_argument('-v', '--verbose', action='store_true', help='Print debug information')
    parser.add_argument('-l', '--level', choices=['CRITICAL', 'FATAL', 'ERROR', 'WARN', 'INFO', 'DEBUG'], 
        default='INFO', help='Logging level')

    args = parser.parse_args()

    logging.basicConfig(level=getattr(logging, args.level))

    # if args.verbose:
    #     logging.basicConfig(level=logging.DEBUG)
    # else:
    #     logging.basicConfig(level=logging.INFO)

    zapi = ZabbixAPI(args.zbx_server, timeout=args.timeout)
    zapi.validate_certs = False
    zapi.login(user=args.zbx_user, password=args.zbx_passwd)

    setattr(args, 'zapi', zapi)

    if hasattr(args, "handler"):
        args.handler(args)

    zapi.logout()
    

if __name__ == "__main__":
    exit(main())



#!/bin/env python3
# coding:utf-8
"""
    Mgnt Servers主机组内主机与其他主机组做序列号匹配,
    匹配上的把普通主机的inventory OOB IP address的值
    设置为 Mgnt Servers主机 snmp interface的地址
"""

import argparse
import logging
from zabbix_api import ZabbixAPIException

parser = argparse.ArgumentParser("Matching inventory OOB IP address")
parser.add_argument('-g', '--group_name', help='Specify a group name')
parser.set_defaults(handler=lambda args: main(args))

target_group = 'Mgnt_servers'

def main(args):

    zapi = args.zapi

    # 1.取所有的常规组, 即非自动发现的组
    plain_groups = zapi.hostgroup.get({
        'output': ['name'],
        'filter': {'flag': 0}
    })


    # 2.取 Mgnt_servers 组的序列号 对应的 oob_ip

    oob_hosts_results = {}

    try:
        target_gid = next(filter(lambda x:x['name'] == target_group, plain_groups))['groupid']
    except StopIteration:
        logging.error(f'未找到组{target_group}')
        exit(1)

    target_hosts = zapi.host.get({
        'output': ['hostid'],
        'selectInventory': ['serialno_a'],
        'selectInterfaces': ['type', 'main', 'ip'],
        'groupids': target_gid
    })

    for host in target_hosts:

        # 抓取'Mgnt servers'组内主机, type=2, main=1的接口ip
        oob_ip = ''.join([item['ip'] for item in filter(lambda i: i['type'] == '2' and i['main'] == '1', host['interfaces'])])
        serialno = host['inventory'].get('serialno_a')

        if oob_ip and serialno:
            oob_hosts_results[serialno] = oob_ip



    # 3. 获取除 Mgnt_servers 外的剩余组内 host, 匹配序列号

    if args.group_name: # 指定仅更新某个组
        other_grps = filter(lambda x:x['name'] == args.group_name, plain_groups)
    else:
        other_grps = filter(lambda x:x['name'] != target_group, plain_groups)
    other_grpids = [g['groupid'] for g in other_grps]
  
    hosts = zapi.host.get({
        'output': ['hostid', 'name'],
        'selectInventory': ['serialno_a'],
        'groupids': other_grpids,
        'withInventory': True
    })

    for host in hosts:

        serialno = host['inventory'].get('serialno_a')
        oob_ip = oob_hosts_results.get(serialno)

        # 匹配到序列号, 更新该host的inventory oob_ip字段
        if serialno and oob_ip:
            try:
                zapi.host.update({
                    'hostid': host['hostid'],
                    'inventory': {
                        'oob_ip': oob_ip
                    }
                })
                logging.info(f'update host:{host["name"]} success, oob_ip -> {oob_ip}')
            except ZabbixAPIException as err:
                logging.error(err)



#!/bin/env python3
# coding:utf-8
"""
    配置主机组的联系人信息，更新到主机组内主机的inventory
"""
import json
import argparse
import ldap3
import logging

parser = argparse.ArgumentParser('Access ldap server and update zabbix host inventory poc')
parser.add_argument('-c', '--contacts-file', required=True, help='HostGroup contacts file')
parser.add_argument('-l', '--ldap-server', required=True, help='ldap server ip address')
parser.add_argument('-o', '--ldap-port', default=389, help='ldap server port')
parser.add_argument('-b', '--ldap-user', required=True, help='ldap bind user')
parser.add_argument('-w', '--ldap-password', required=True, help='ldap password')
parser.set_defaults(handler=lambda args: main(args))

class LdapServer():
    """
    使用 ldap3 模块连接ldap server, 通过 dn 查询用户对应的属性
    """
    def __init__(self, host, port, bind_user, password):
        self.__server = ldap3.Server(host=host, port=port)
        self.__conn = ldap3.Connection(self.__server, bind_user, password, auto_bind=True)

    def get_user_info(self, dn):
        if not dn:
            return {}
        res = self.__conn.search(
            search_base=dn,
            search_filter='(objectClass=*)',
            search_scope=ldap3.BASE,
            attributes=ldap3.ALL_ATTRIBUTES  # 该参数无法取消, 取消后不返回任何属性
        )
        if res:
            return self.__conn.response[0].get('attributes')
        return {}


def main(args):
    """
    读取 Contacts.json 文件中 HostGroup 联系人信息, 按 GroupName 升序排列
    遍历 HostGroup 并将 zabbix 中对应的 Host 更新 Poc 信息。
    """

    contacts_file = args.contacts_file
    contacts = {}

    # 读取文件中 HostGroup 联系人信息, 生成contacts, [{group1's info}, {group2's info}, ...]
    with open(contacts_file, 'r', encoding='utf8')as fp:
        temp = json.load(fp)
        for info in temp['HostGroup']:
            contacts[info['GroupName']] = info

    zapi = args.zapi

    zbx_groups = zapi.hostgroup.get({
        'output': ['groupid', 'name'],
        'selectHosts': ['hostid'],
        'filter': {'name': list(contacts.keys())}
    })

    # 登录ldap server
    ldap = LdapServer(host=args.ldap_server,
                      port=args.ldap_port,
                      bind_user=args.ldap_user,
                      password=args.ldap_password)

    # 将zbx_groups 按照 group name 升序排列
    zbx_groups.sort(key=lambda g: g.get('name'))

    for zbx_group in zbx_groups:
        contact = contacts.get(zbx_group.get('name'), {})
        poc_1_dn = contact.get('poc_1_dn')
        poc_2_dn = contact.get('poc_2_dn')
        ldap.get_user_info(poc_1_dn)
        ldap.get_user_info(poc_2_dn)
        poc_1_info = ldap.get_user_info(poc_1_dn)
        poc_2_info = ldap.get_user_info(poc_2_dn)

        zapi.host.massupdate({
            'hosts': zbx_group.get('hosts'),
            'inventory_mode': 1,  # 1 - Automatic
            'inventory': {
                'poc_1_name': ''.join(poc_1_info.get('sn', '') + poc_1_info.get('givenName', '')),
                'poc_1_email': ','.join(poc_1_info.get('mail', '')),
                'poc_1_phone_a': poc_1_info.get('telephoneNumber', [''])[0],
                'poc_1_phone_b': poc_1_info.get('telephoneNumber', [''])[-1],
                'poc_1_cell': ','.join(poc_1_info.get('mobile', '')),
                'poc_1_screen': ','.join(poc_1_info.get('uid', '')),
                'poc_1_notes': '',  # ldap暂无设置此属性
                'poc_2_name': ''.join(poc_2_info.get('sn', '') + poc_2_info.get('givenName', '')),
                'poc_2_email': ','.join(poc_2_info.get('mail', '')),
                'poc_2_phone_a': poc_2_info.get('telephoneNumber', [''])[0],
                'poc_2_phone_b': poc_2_info.get('telephoneNumber', [''])[-1],
                'poc_2_cell': ','.join(poc_2_info.get('mobile', '')),
                'poc_2_screen': ','.join(poc_2_info.get('uid', '')),
                'poc_2_notes': ''  # ldap暂无设置此属性
            }
        })
        logging.info(f"update success! HostGroup-> [{zbx_group.get('name')!r}] ")



import re
import json
import argparse
import logging
import pandas as pd
import numpy as np
import IPy
from zabbix_api import ZabbixAPIException
from openpyxl.utils import get_column_letter

parser = argparse.ArgumentParser('find ip from host inventory')
parser.add_argument('-f', '--check_file', required=True, help='a list of ip range of each IDC')
parser.add_argument('--dump', choices=['console', 'excel'], default='console', help='Print to screen, or save to excel')
group = parser.add_mutually_exclusive_group()  # 互斥参数组, 组内参数不能同时出现
group.add_argument('--check_agent', action='store_true', help='display invalid interface')
group.add_argument('--delete_invalid', action='store_true', help='delete invalid interface')
group.add_argument('--add_extra', action='store_true', help='add extra ip to interface')
group.add_argument('--delete_extra', action='store_true', help='delete extra ip from interface')
parser.set_defaults(handler=lambda args: main(args))

# 匹配windows和linux ip的正则表达式
win_host_ptn = re.compile(r'windows', re.I)
linux_host_ptn = re.compile(r'linux', re.I)
linux_ip_ptn = re.compile(r'((?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))(?:\/(?:\d|[1-2]\d|3[0-2]))', re.M)
win_ip_ptn = re.compile(r'((?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))', re.M)


# 各机房网段信息
IDC_NETWORKS = []

def main(args):

    zapi = args.zapi

    # 读取ip_range.json各机房ip网段
    global IDC_NETWORKS
    with open(args.check_file, 'r', encoding='utf8')as fp:
        for _, networks in json.load(fp).items():
            IDC_NETWORKS +=  [IPy.IP(net) for net in networks]


    zbx_host = zapi.host.get({
        'output':['name', 'hostid', 'proxy_hostid'],
        'selectInventory':['inventory_mode', 'location', 'host_networks', 'os_full', 'os_short'],
        'selectInterfaces':['interfaceid', 'ip', 'type', 'main'],
       #'filter': {'host':'1.1.1.3'},
    })



    # 生成pandas数据表, 用来输出屏幕和保存文件
    df = pd.DataFrame(columns=['zbx_host', 'os', 'location', 'agent_interfaces', 'other_ipv4s'])


    for host in zbx_host:

        location = host['inventory'].get('location')
        zbx_host = host['name']
        os_short= host['inventory'].get('os_short', '')

        # 指定了--delete_invalid, 删除主机中无效的agent interfaces
        if args.delete_invalid:
            delete_invaild(zapi, host)

        # 指定了--delete_extra, 只保留第一个agent interfaces，其余的全部删除
        if args.delete_extra:
            delete_extra(zapi, host)


        # 指定了--add_extra, 将额外的主机地址添加到host agent interface
        if args.add_extra:
            add_extra(zapi, host)


        # 指定了--check_agent, 过滤出IP地址不存在于主机资产的agent interface
        if args.check_agent:
            agent_ips = check_agent(host)
            if not agent_ips:
                logging.debug(f"All agent interfaces of host->{host['name']} are in host_networks")
                continue
        else:
            agent_ips = get_agent_ips(host)

        other_ips = get_other_ips(host)

        # 添加数据到数据表
        df.loc[len(df)] = [zbx_host, os_short, location, ','.join(agent_ips), ','.join(other_ips)]

    # 将结果按location排序
    res = df.sort_values(by=['location'], na_position='last').reset_index(drop=True)
    if res.empty:
        logging.info('No data retrieved.')
        exit()

    if args.dump == 'excel':
        save_to_excel(res, 'result.xlsx')
    elif args.dump == 'console':
        # 设置屏幕输出列对齐
        pd.set_option('display.width', 180)
        pd.set_option('display.max_rows', None)
        pd.set_option('display.max_columns', 99)
        pd.set_option('display.unicode.ambiguous_as_wide', True)
        pd.set_option('display.unicode.east_asian_width', True)
        print(res)




def to_excel_auto_column_weight(df, writer, sheet_name="Sheet1"):
    """
    DataFrame保存为excel并自动设置列宽
    """
    df.to_excel(writer, sheet_name=sheet_name, index=False)

    column_widths = (
        df.columns.to_series().apply(lambda x: len(x.encode('utf8'))).values
    )
    max_widths = (
        df.astype(str).applymap(lambda x: len(x.encode('utf8'))).agg(max).values
    )

    widths = np.max([column_widths, max_widths], axis=0)
    worksheet = writer.sheets[sheet_name]
    for i, width in enumerate(widths, 1):
        worksheet.column_dimensions[get_column_letter(i)].width = width + 2


def valid_ip(ip):
    """
    验证ip是否在各机房网段内
    """
    global IDC_NETWORKS

    if ip and IDC_NETWORKS:
        for net in IDC_NETWORKS:
            if ip in net:
                return True
    return False


def get_agent_ips(host) -> list:
    """
    获取 zabbix agent interfaces ip
    """
    agent_interfaces = [inf for inf in host['interfaces'] if inf['type'] == '1']
    return [inf['ip'] for inf in agent_interfaces]


def get_host_networks(host) -> set:
    """
    获取inventory中host_networks里面的ip地址
    """
    global win_host_ptn, linux_host_ptn, win_ip_ptn, linux_ip_ptn

    os_full = host['inventory'].get('os_full', '')
    if win_host_ptn.search(os_full):
        host_networks = set(win_ip_ptn.findall(host['inventory'].get('host_networks','')))
    elif linux_host_ptn.search(os_full):
        host_networks = set(linux_ip_ptn.findall(host['inventory'].get('host_networks','')))
    else:
        host_networks = set()  # 无法判断windows、linux
    return host_networks


def get_other_ips(host) -> list:
    """
    inventory 中 host_networks里面的ip 排除掉agent_ips,
    再排除掉不在IDC_NETWORKS网段内的ip
    """
    agent_ips = get_agent_ips(host)
    host_networks = get_host_networks(host)
    other_ips = list(host_networks - set(agent_ips))
    return [ip for ip in other_ips if valid_ip(ip)]


def delete_invaild(zapi, host) -> None:
    """
    删除ip不在 host_networks中的非第一个agent interface
    """
    host_networks = get_host_networks(host)
    if not host_networks:
        return

    for inf in host['interfaces'][::-1]:
        if inf['type'] == '1' and inf['main'] != '1':
            if inf['ip'] in host_networks:
                continue
            try:
                zapi.hostinterface.delete([inf['interfaceid']])
                logging.info(f"Invalid agent interface deleted: host->{host['name']}, agent_ip->{inf['ip']}")
                host['interfaces'].remove(inf)
            except ZabbixAPIException:
                logging.error(f"Delete agent interface failed: host->{host['name']}, agent_ip->{inf['ip']}")


def delete_extra(zapi, host) -> None:
    """
    指定了--delete_extra, 只保留第一个agent interfaces，其余的全部删除
    """
    for inf in host['interfaces'][::-1]:
        if inf['type'] == '1' and inf['main'] != '1':
            try:
                zapi.hostinterface.delete([inf['interfaceid']])
                logging.info(f"Extra agent interface deleted: host->{host['name']}, agent_ip->{inf['ip']}")
                host['interfaces'].remove(inf)
            except ZabbixAPIException:
                logging.error(f"Delete agent interface failed: host->{host['name']}, agent_ip->{inf['ip']}")


def add_extra(zapi, host) -> None:
    """
    将额外的主机地址添加到host agent interface
    """
    for ip in get_other_ips(host):
        try:
            zapi.hostinterface.create({
                'hostid': host['hostid'],
                'dns': '',
                'ip': ip,
                'main': 0,
                'port': '10050',
                'type': 1,
                'useip': 1
            })
            logging.info(f"Successfully added interface: host->{host['name']}, extra_ip->{ip}")
            host['interfaces'].append({'main': '0', 'type': '1', 'ip': ip})
        except ZabbixAPIException:
            logging.error(f"Add interface faild! host->{host['name']},extra_ip->{ip}")


def check_agent(host) -> list:
    """
    打印agent interface IP地址不存在于主机资产的host_networks的信息
    """
    host_networks = get_host_networks(host)
    if not host_networks: # host_networks未匹配出ip
        logging.debug(f"host->{host['name']} has no host_networks, skipped.")
        return []

    agent_ips = get_agent_ips(host)
    return [ip for ip in agent_ips if  ip not in host_networks]


def save_to_excel(df: pd.DataFrame, fname: str):
    if not fname.endswith('.xlsx'):
        fname = fname + '.xlsx'

    writer = pd.ExcelWriter(fname)
    to_excel_auto_column_weight(df, writer)
    writer.save()
    logging.info(f"Successfully saved file to {fname}")
